use std::str::FromStr;

use thiserror::Error;

#[derive(Debug, PartialEq, Eq)]
pub struct RangeMapping {
    source: u32,
    target: u32,
    range: u32,
}

#[derive(Debug, Error)]
pub enum ParseErrorRangeMapping {
    #[error("Needs to have at least 3 numbers")]
    LessThan3Numbers,
    #[error("Every item must be a valid unsigned number")]
    InvalidFormatForNumbers,
}
impl FromStr for RangeMapping {
    type Err = ParseErrorRangeMapping;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let mut numbers_as_3 = s.split(" ").take(3).map(|unparsed| {
            unparsed
                .parse::<u32>()
                .map_err(|_| ParseErrorRangeMapping::InvalidFormatForNumbers)
        });
        let (target, source, range) = (
            numbers_as_3
                .next()
                .ok_or(ParseErrorRangeMapping::LessThan3Numbers)??,
            numbers_as_3
                .next()
                .ok_or(ParseErrorRangeMapping::LessThan3Numbers)??,
            numbers_as_3
                .next()
                .ok_or(ParseErrorRangeMapping::LessThan3Numbers)??,
        );

        Ok(Self {
            target,
            source,
            range,
        })
    }
}

impl RangeMapping {
    pub fn source(&self) -> u32 {
        self.source
    }

    pub fn target(&self) -> u32 {
        self.target
    }

    pub fn range(&self) -> u32 {
        self.range
    }
}
#[cfg(test)]
mod testing {
    use super::*;

    #[test]
    fn parse_from_line() {
        const INPUT: &str = "50 98 2";
        let expected = RangeMapping {
            source: 98,
            target: 50,
            range: 2,
        };
        let actual: RangeMapping = INPUT.parse().unwrap();
        assert_eq!(expected, actual);
    }
}
